Imports System.Math

Public Class Form1
    Inherits System.Windows.Forms.Form

#Region " Windows Form Designer generated code "

    Public Sub New()
        MyBase.New()

        'This call is required by the Windows Form Designer.
        InitializeComponent()

        'Add any initialization after the InitializeComponent() call

    End Sub

    'Form overrides dispose to clean up the component list.
    Protected Overloads Overrides Sub Dispose(ByVal disposing As Boolean)
        If disposing Then
            If Not (components Is Nothing) Then
                components.Dispose()
            End If
        End If
        MyBase.Dispose(disposing)
    End Sub

    'Required by the Windows Form Designer
    Private components As System.ComponentModel.IContainer

    'NOTE: The following procedure is required by the Windows Form Designer
    'It can be modified using the Windows Form Designer.  
    'Do not modify it using the code editor.
    Friend WithEvents picCanvas As System.Windows.Forms.PictureBox
    Friend WithEvents MainMenu1 As System.Windows.Forms.MainMenu
    Friend WithEvents MenuItem1 As System.Windows.Forms.MenuItem
    Friend WithEvents mnuFullScale As System.Windows.Forms.MenuItem
    <System.Diagnostics.DebuggerStepThrough()> Private Sub InitializeComponent()
        Me.picCanvas = New System.Windows.Forms.PictureBox
        Me.MainMenu1 = New System.Windows.Forms.MainMenu
        Me.MenuItem1 = New System.Windows.Forms.MenuItem
        Me.mnuFullScale = New System.Windows.Forms.MenuItem
        Me.SuspendLayout()
        '
        'picCanvas
        '
        Me.picCanvas.Dock = System.Windows.Forms.DockStyle.Fill
        Me.picCanvas.Location = New System.Drawing.Point(0, 0)
        Me.picCanvas.Name = "picCanvas"
        Me.picCanvas.Size = New System.Drawing.Size(292, 273)
        Me.picCanvas.TabIndex = 0
        Me.picCanvas.TabStop = False
        '
        'MainMenu1
        '
        Me.MainMenu1.MenuItems.AddRange(New System.Windows.Forms.MenuItem() {Me.MenuItem1})
        '
        'MenuItem1
        '
        Me.MenuItem1.Index = 0
        Me.MenuItem1.MenuItems.AddRange(New System.Windows.Forms.MenuItem() {Me.mnuFullScale})
        Me.MenuItem1.Text = "&Scale"
        '
        'mnuFullScale
        '
        Me.mnuFullScale.Index = 0
        Me.mnuFullScale.Text = "&Full Scale"
        '
        'Form1
        '
        Me.AutoScaleBaseSize = New System.Drawing.Size(5, 13)
        Me.ClientSize = New System.Drawing.Size(292, 273)
        Me.Controls.Add(Me.picCanvas)
        Me.Menu = Me.MainMenu1
        Me.Name = "Form1"
        Me.Text = "Mandelbrot"
        Me.ResumeLayout(False)

    End Sub

#End Region

#Region "Variables"
    ' Variables for rubberband drawing.
    Private m_DrawingBox As Boolean
    Private m_X1, m_Y1, m_X2, m_Y2 As Integer
    Private m_ZoomingBitmap As Bitmap
    Private m_ZoomingGraphics As Graphics

    ' Drawing variables.
    Private m_Bitmap As Bitmap
    Private m_Graphics As Graphics

    ' World coordinates.
    Private Const MIN_X As Double = -2.2
    Private Const MAX_X As Double = 1
    Private Const MIN_Y As Double = -1.2
    Private Const MAX_Y As Double = 1.2
    Private m_Wxmin As Double = MIN_X
    Private m_Wxmax As Double = MAX_X
    Private m_Wymin As Double = MIN_Y
    Private m_Wymax As Double = MAX_Y

    ' Calculation values.
    Public Const MaxIterations As Integer = 128

    ' Colors.
    Public m_NumColors As Integer
    Private m_Colors() As Color
#End Region ' Variables

    ' Draw the first image.
    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' Load the colors.
        m_NumColors = 15
        ReDim m_Colors(m_NumColors)
        For i As Integer = 0 To m_NumColors - 1
            m_Colors(i) = Color.FromArgb( _
                QBColor(i + 1) + &HFF000000)
        Next i

        ' Draw at the current (full) scale.
        DrawMandelbrot()
    End Sub

    ' Redraw with the new form size.
    Private Sub Form1_Resize(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Resize
        ' If we haven't loaded the colors yet, load them.
        If m_Colors Is Nothing Then
            m_NumColors = 15
            ReDim m_Colors(m_NumColors)
            For i As Integer = 0 To m_NumColors - 1
                m_Colors(i) = Color.FromArgb( _
                    QBColor(i + 1) + &HFF000000)
            Next i
        End If

        ' If the form is minimized, do nothing.
        If Me.WindowState = FormWindowState.Minimized Then Exit Sub

        ' See if we just moved and didn't resize.
        If Not (m_Bitmap Is Nothing) Then
            ' We have a Bitmap. 
            ' If it's already the right size, do nothing.
            If m_Bitmap.Width = picCanvas.ClientSize.Width And _
               m_Bitmap.Height = picCanvas.ClientSize.Height _
                Then Exit Sub
        End If

        ' Draw at the current scale.
        DrawMandelbrot()
    End Sub

#Region "Rubberbanding"
    ' Start rubberband drawing.
    Private Sub picCanvas_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles picCanvas.MouseDown
        m_DrawingBox = True
        m_X1 = e.X
        m_Y1 = e.Y
        m_X2 = m_X1
        m_Y2 = m_Y1

        ' Make a copy of the current bitmap.
        m_ZoomingBitmap = New Bitmap(m_Bitmap)
        m_ZoomingGraphics = Graphics.FromImage(m_ZoomingBitmap)
    End Sub

    ' Continue rubberband drawing.
    Private Sub picCanvas_MouseMove(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles picCanvas.MouseMove
        If Not m_DrawingBox Then Exit Sub

        ' Save the new corner.
        m_X2 = e.X
        m_Y2 = e.Y

        ' Draw the new box.
        Dim gr As Graphics = picCanvas.CreateGraphics()
        Dim rect As New Rectangle
        rect.X = Min(m_X1, m_X2)
        rect.Y = Min(m_Y1, m_Y2)
        rect.Width = Abs(m_X2 - m_X1)
        rect.Height = Abs(m_Y2 - m_Y1)
        m_ZoomingGraphics.DrawImage(m_Bitmap, 0, 0)
        m_ZoomingGraphics.DrawRectangle(Pens.White, rect)

        ' Display the result.
        picCanvas.Image = m_ZoomingBitmap
    End Sub

    ' Finish rubberband drawing.
    Private Sub picCanvas_MouseUp(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles picCanvas.MouseUp
        If Not m_DrawingBox Then Exit Sub
        m_DrawingBox = False

        ' Dispose of the zooming Bitmap and Graphics.
        picCanvas.Image = Nothing
        m_ZoomingBitmap.Dispose()
        m_ZoomingGraphics.Dispose()
        m_ZoomingBitmap = Nothing
        m_ZoomingGraphics = Nothing

        ' Save the new corner.
        m_X2 = e.X
        m_Y2 = e.Y

        ' Put the selected coordinates in order.
        Dim x1, x2, y1, y2, factor As Double
        x1 = Min(m_X1, m_X2)
        x2 = Max(m_X1, m_X2)
        y1 = Min(m_Y1, m_Y2)
        y2 = Max(m_Y1, m_Y2)

        ' Convert screen coords into drawing coords.
        factor = (m_Wxmax - m_Wxmin) / picCanvas.ClientSize.Width
        m_Wxmax = m_Wxmin + x2 * factor
        m_Wxmin = m_Wxmin + x1 * factor

        factor = (m_Wymax - m_Wymin) / picCanvas.ClientSize.Height
        m_Wymax = m_Wymin + y2 * factor
        m_Wymin = m_Wymin + y1 * factor

        ' Draw the new Mandelbrot set.
        DrawMandelbrot()
    End Sub
#End Region ' Rubberbanding

    ' Adjust the aspect ratio of the selected
    ' coordinates so they fit the window properly.
    Private Sub AdjustAspect()
        Dim want_aspect, picCanvas_aspect As Double
        Dim hgt, wid, mid As Double

        want_aspect = (m_Wymax - m_Wymin) / (m_Wxmax - m_Wxmin)
        picCanvas_aspect = picCanvas.ClientSize.Height / picCanvas.ClientSize.Width
        If want_aspect > picCanvas_aspect Then
            ' The selected area is too tall and thin.
            ' Make it wider.
            wid = (m_Wymax - m_Wymin) / picCanvas_aspect
            mid = (m_Wxmin + m_Wxmax) / 2
            m_Wxmin = mid - wid / 2
            m_Wxmax = mid + wid / 2
        Else
            ' The selected area is too short and wide.
            ' Make it taller.
            hgt = (m_Wxmax - m_Wxmin) * picCanvas_aspect
            mid = (m_Wymin + m_Wymax) / 2
            m_Wymin = mid - hgt / 2
            m_Wymax = mid + hgt / 2
        End If
    End Sub

    ' Draw the Mandelbrot set.
    Private Sub DrawMandelbrot()
        ' Do nothing if the canvas has zero size.
        If picCanvas.ClientSize.Width < 1 Or _
           picCanvas.ClientSize.Height < 1 Then Exit Sub

        ' Create the Bitmap and Graphic objects.
        If Not (m_Graphics Is Nothing) Then
            picCanvas.Image = Nothing
            m_Graphics.Dispose()
            m_Bitmap.Dispose()
        End If
        m_Bitmap = New Bitmap(picCanvas.ClientSize.Width, picCanvas.ClientSize.Height)
        m_Graphics = Graphics.FromImage(m_Bitmap)

        ' Clear the bitmap.
        m_Graphics.Clear(Color.Black)
        picCanvas.Image = m_Bitmap
        picCanvas.Refresh()

        ' Work until the magnitude squared > 4.
        Const MAX_MAG_SQUARED As Double = 4.0

        ' Adjust the coordinate bounds to fit picCanvas.
        AdjustAspect()

        Dim clr As Integer
        Dim ReaC, ImaC, dReaC, dImaC, ReaZ, ImaZ, ReaZ2, ImaZ2 As Double

        ' dReaC is the change in the real part
        ' (X value) for C. dImaC is the change in the
        ' imaginary part (Y value).
        Dim wid As Integer = picCanvas.ClientSize.Width
        Dim hgt As Integer = picCanvas.ClientSize.Height
        dReaC = (m_Wxmax - m_Wxmin) / (wid - 1)
        dImaC = (m_Wymax - m_Wymin) / (hgt - 1)

        ' Calculate the values.
        ReaC = m_Wxmin
        For i As Integer = 0 To wid - 1
            ImaC = m_Wymin
            For j As Integer = 0 To hgt - 1
                ReaZ = 0
                ImaZ = 0
                ReaZ2 = 0
                ImaZ2 = 0
                clr = 0
                Do While clr < MaxIterations And _
                        ReaZ2 + ImaZ2 < MAX_MAG_SQUARED
                    ' Calculate Z(clr).
                    ReaZ2 = ReaZ * ReaZ
                    ImaZ2 = ImaZ * ImaZ
                    ImaZ = 2 * ImaZ * ReaZ + ImaC
                    ReaZ = ReaZ2 - ImaZ2 + ReaC
                    clr = clr + 1
                Loop

                ' Set the pixel's color.
                m_Bitmap.SetPixel(i, j, _
                    m_Colors(clr Mod m_NumColors))

                ' Move to the next point.
                ImaC = ImaC + dImaC
            Next j
            ReaC = ReaC + dReaC

            ' Let the user know we're not dead.
            If i Mod 10 = 0 Then picCanvas.Refresh()
        Next i

        ' Update the image.
        picCanvas.Refresh()

        Me.Text = "Mandelbrot (" & _
            Format$(m_Wxmin) & ", " & _
            Format$(m_Wymin) & ")-(" & _
            Format$(m_Wxmax) & ", " & _
            Format$(m_Wymax) & ")"
    End Sub

    ' Return to full scale.
    Private Sub mnuFullScale_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuFullScale.Click
        m_Wxmin = MIN_X
        m_Wxmax = MAX_X
        m_Wymin = MIN_Y
        m_Wymax = MAX_Y

        DrawMandelbrot()
    End Sub

End Class
